<?php
// $Id: imageset.inc,v 1.5 2008/12/20 22:47:37 pvb Exp $

/**
* output themed imageset
* @param $name - imageset name or format 'name:index' index = NULL
*  @param $index - index in imageset
*/
function theme_imageset($name, $index = NULL, $attributes = NULL) {

  if (!isset($index)) {
    if (preg_match('/(\w+):(\d+)/', $name, $matches)) {
      $name = $matches[1];
      $index = $matches[2];
    }
  }    
  $img = _imageset_imageset($name, TRUE);
  $output = '';
  if (isset($img) && !$img['dis']) {		
    $outname = _imageset_selector_outname();
    if ($attributes && is_array($attributes)) {
      if ($attributes['outname']) {
        $outname = $attributes['outname'];
        unset($attributes['outname']);
      }
    }
    if ($img['fixpngie']) {
      $attributes['class'] .= ' is-pngfixiebk';
    }
    $output = module_invoke_all('imageset_element', 'out-'.$outname, $name, $index, $attributes);
    if (is_array($output)) $output = reset($output);  
  }
  return $output;
}

/*
arrray(
  [version] => version number
  [list] => array()
  )
*/
function imageset_get_settings() {

  $settings = variable_get('imageset', array());
  if ( (is_array($settings)) and (isset($settings['version']) && !is_array($settings['version'])) ) {
    $version_vars = $settings['version'];
    $result = $settings['list'];
  } 
  else {
    $version_vars = 0;
    $result = $settings;
  }
  if ($version_vars != VERSION_IMAGESET_VAR) {
    imageset_update_version();
    imageset_set_settings($result);
  }
  return $result;
}

function imageset_set_settings($list = array()) {
  $settings = array('version' => VERSION_IMAGESET_VAR, 'list' => $list);
  variable_set('imageset', $settings);
}

function imageset_update_version() {
  imageset_clear_css();
}

function imageset_names() {
	
  $settings = imageset_get_settings();
  foreach($settings as $name=>$value) {
    $result[$name] = $name;
  }
  return $result;
}

function _imageset_selector_outname($value = NULL) {

  return (isset($value) && $value['outname']? $value['outname']: 'default');
}

/*
$size - string "WxH"
return array ('sizex' => W, 'sizey' => H)
*/
function _imageset_explode_size($size) {

  $v = explode('x', $size);
  return array(
    'sizex' => intval($v[0]),
	'sizey' => intval($v[1]),
  );
}		

/*
return array
['imgname']	- path to imageset
['sizex'] 	- width map
['sizey'] 	- height map
['vh'] 	- orientation
['dis']            - disabled
['fixpngie']    - fix alpha chanel for IE
*/
function _imageset_imageset($name, $check_css = FALSE) {
global $_imageset_imagesets;

  if (!isset($_imageset_imagesets)) {
    $_imageset_imagesets = array();
  }
  if (!isset($_imageset_imagesets[$name])) {
    $v = imageset_get_settings();
    $_imageset_imagesets[$name] = $v[$name];
  }
  $img = $_imageset_imagesets[$name];
  // include CSS
  if ($img && !$img['check_css'] && $check_css) {
    if (!$img['dis']) {
      imageset_check_css($name);
      if ($img['fixpngie']) imageset_init_pngfixie();
    }
    $_imageset_imagesets[$name]['check_css'] = TRUE;
  }
  return $img;
}

function imageset_init_pngfixie() {
static $icon_js;

  if (!isset($icon_js)) {
    $icon_js = TRUE;    
    $path_ispngfixbk_js = drupal_get_path('module', 'imageset') . '/ispngfixbk.js';
    drupal_set_html_head("<!--[if lt IE 7]> ".
      "<style type=\"text/css\">img.is-pngfixiebk{filter:expression(IS_PNGFIX.fix(this))}</style>".
      "<script type=\"text/javascript\" src=\"". base_path() . $path_ispngfixbk_js. "\"></script>".
      " <![endif]-->");
  }
}

function _imageset_imagesets_uses($all = FALSE) {
global $_imageset_imagesets;
	
  $result = array();
  $v = imageset_get_settings();
  foreach($v as $name => $img) {
    if(!$img['dis']) {
      if ($all || $img['persistent'] || $_imageset_imagesets[$name]) {
        $result[$name] = $img;
      }
    }
  }
  return $result;
}

/*
NULL - delete
*/
function _imageset_imageset_update($name, $newvalue = NULL) {
global $_imageset_imagesets;

  $v = imageset_get_settings();
  $save_imgname = $v[$name]['imgname'];
  if (!isset($newvalue['name']) || (isset($newvalue['name']) && $name != $newvalue['name'])) {  // change name		
    unset($v[$name]);
  }
// reset cashe	
  unset($_imageset_imagesets[$name]);
	
  if (isset($newvalue['name'])) {
    $name = $newvalue['name'];
    unset($newvalue['name']);
  }
  if (isset($newvalue)) {
    $v[$name] = $newvalue;
  }
  ksort($v);
  imageset_set_settings($v);
	
  if ($save_imgname && !isset($newvalue)) {
    if (dirname($save_imgname) == file_create_path('imagesets')) {
      file_delete($save_imgname);
    }
  }
}


function _imageset_selector_npp($npp) {

  $v = variable_get('imageset_style', array());
  return $v[intval($npp)-1];
}

function _imageset_selectors($name = NULL) {
	
  $v = variable_get('imageset_style', array());
  if ($name) {
    $result = array();
    foreach($v as $style) {
      if ($style['name'] == $name) {
        $result[] = $style;
      }
    }
    if (count($result) == 0) {	// default
      $result[] = array(
        'name' => $name,
        'select' => 1,
        'css' => '',
        'outname' => 'default',
      );
    }
    return $result;
  } 
  else return $v;
}

function _sort_imageset_selector($val1, $val2) {

  $result = strcasecmp($val1['name'], $val2['name']);
  if ($result == 0) {
    $result= ($val1['weight'] < $val2['weight'])? -1: (($val1['weight'] > $val2['weight'])? 1: 0);
  }
  if ($result == 0) {
    $result = strcasecmp($val1['css'], $val2['css']);
  }
  return $result;
}
/*
value - array(
	'name' - name imageset
	'select' - index for select 1...n
	'css' 		- CSS selector prefix
	'outname' 		- Outer name
	'weight'	- weight position in CSS
	)
if (value == NULL) DELETE
*/
function _imageset_selector_update($npp, $value = NULL) {

  if (!$npp && !isset($value))  return;
	
  $v = variable_get('imageset_style', array());
  if ($npp) {
    if (isset($value)) {
      $v[$npp - 1] = $value;
    }
    else unset($v[$npp - 1]);
  } 
  else $v[] = $value;
  usort($v, _sort_imageset_selector);
  variable_set('imageset_style', $v);
}

function _imageset_selector_delete($name) {

  $v = variable_get('imageset_style', array());
  foreach($v as $npp => $value) {
    if ($value['name'] == $name) {unset($v[$npp]);}
  }
  variable_set('imageset_style', $v);
}

function install_imageset($modulename, $filename_dat, &$buffer = NULL) {
  
  imageset_clear_css();
// first string - description
// name, filename, WxH, orientation,default outname; CSS selectors
  $file = fopen($filename_dat, 'r');
  if ($file) {
    $data = fgetcsv($file, 0, ';');
    $items = explode(',', $data[0]);
    $name = $items[0];
    $imgname = $items[1];
    if (dirname($imgname) == '' || dirname($imgname) == '.') {
      $dir = dirname($filename_dat);
      if ($dir == '') $dir = getcwd ();
      $imgname = $dir.'/'.$imgname;
    }
    $value = array_merge(
      array(
        'imgname' => $imgname,
        'vh' => intval($items[3]),
        'module' => $modulename,
      ),
      _imageset_explode_size($items[2])
    );
    $replace_index = 0;
    while (_imageset_imageset($name)) {
      if(!$replace_index) {
        if (preg_match("/_(\d)+$/", $name, $matches)) {
          $replace_index = $matches[1];
        } 
        else $name .= '_0';
      }
      $replace_index++;
      $name = preg_replace("/_(\d)+$/", '_'.$replace_index, $name);
    }
    _imageset_imageset_update($name, $value);
    $outname = $items[4];
    $select = 1;
    while ($select< count($data)) {
      $value = array(
        'name' => $name,
        'select' => $select,
        'css' => $data[$select],
        'outname' => $outname,
        'weight' => $select - 1,
      );
      _imageset_selector_update(0, $value);
      $select++;
    }
    if (func_num_args()>2) { 
      $buffer = array();
      while ($buffer[] = fgets($file)) { }
    }
    fclose($file);
  }
  return $name;
}

function uninstall_imageset($modulename) {

  imageset_clear_css();
  $v = imageset_get_settings();
  foreach($v as $name => $img) {
    if ($img['module'] == $modulename) {
      _imageset_imageset_update($name);
      _imageset_selector_delete($name);
    }
  }
}

function imageset_clear_css() {
  $v = variable_get('imageset_css', array());
  foreach($v as $name_css => $value) {
    file_delete(file_create_path('imagesets').'/'.$name_css);
  }
  variable_del('imageset_css');
  drupal_clear_css_cache();
}

function imageset_check_css($names = NULL) {
static $lock;
  
  if ($lock) return;
  $lock = TRUE;
	
  $exists_css = FALSE;
  if ($names) {
    if (is_array($names)) {
      $imagesets =array();
      foreach($names as $name) {
        $imagesets[$name] = _imageset_imageset($name);
      }
    }
    else {
      $imagesets = array($names => _imageset_imageset($names));
    }
  }
  else {
    $imagesets = _imageset_imagesets_uses();
    ksort($imagesets);
  }
  if (count($imagesets) == 0) return;    
	
  $names = implode(',', array_keys($imagesets));
  $name_css = md5($names).'.css';
  $file_css = file_create_path('imagesets').'/'.$name_css;
  $file_css_fixie = file_create_path('imagesets').'/'.md5($names).'-fixie.css';
  $v = variable_get('imageset_css', array());
  if (isset($v[$name_css])) {
    if (file_exists($file_css)) $exists_css = TRUE;
  }
  if (!$exists_css) {
    file_check_directory(file_create_path('imagesets'), FILE_CREATE_DIRECTORY);
    $css = imageset_prepare_css($imagesets);
    if ($css['css']) {
      file_save_data($css['css'], $file_css, FILE_EXISTS_REPLACE);
    }
    if ($css['css_fixie']) {
      file_save_data($css['css_fixie'], $file_css_fixie, FILE_EXISTS_REPLACE);
    } 
    else {
	  if (file_exists($file_css_fixie)) unlink($file_css_fixie);
	}
    $v[$name_css] = $names;
    variable_set('imageset_css', $v);
  }
  drupal_add_css($file_css);
  if (file_exists($file_css_fixie)) {
    drupal_add_css_ie($file_css_fixie);
  }
  $lock = FALSE;
}

function drupal_add_css_ie($filename) {
    drupal_set_html_head("<!--[if lt IE 7]> ".
    "<style type=\"text/css\" media=\"all\">@import \"".base_path().$filename."\";</style>".
    " <![endif]-->");
}

/*
$imagesets - array imagesets
retirn array(
  'css'
  'css_fixie')
**/
function imageset_prepare_css($imagesets) {

  $css = '';
  $css_fixie = '';
  $save_selectors = array();
  foreach($imagesets as $name => $img) {
    if ($info = image_get_info($img['imgname'])) {
      switch ($img['vh']) {
        case IS_VERTICAL:
          $mi_index = $info['height'] / $img['sizey'];
          $mi_variant = $info['width'] / $img['sizex'];
          break;
        case IS_HORIZONTAL:
          $mi_index = $info['width'] / $img['sizex'];
          $mi_variant = $info['height'] / $img['sizey'];
          break;
      }
      $selectors = array();
      $styles = _imageset_selectors($name);
      foreach ($styles as $style) {
        $variant = $style['select'];
        $outname = _imageset_selector_outname($style);
        $outindex = "$variant-$outname";
        if (!isset($selectors[$outindex])) {
          $selectors[$outindex] = array(
            'variant' => $variant,
            'outname' => $outname,
            'prefix' => explode(',', $style['css']),
          );
          $save_selectors["$name-$outname"] = array('name' => $name, 'outname' => $outname);
        } 
        else {
          $selectors[$outindex]['prefix'] = array_merge($selectors[$outindex]['prefix'], explode(',', $style['css']));
        }
      }
      for ($i = 1; $i <= $mi_index; $i++) {
        foreach($selectors as $outindex => $value) {
          $variant = $value['variant'];
          switch ($img['vh']) {
            case IS_VERTICAL:
              $ofsx = -($img['sizex'] * ($variant - 1));
              $ofsy = -($img['sizey'] * ($i - 1));
              break;
            case IS_HORIZONTAL:
              $ofsx = -($img['sizex'] * ($i - 1));
              $ofsy = -($img['sizey'] * ($variant - 1));
              break;
          }
          $css_selector = '';
          $element = imageset_get_csselement($value['outname'], $name, $i);
          $css_selector_fixie = '';
          $element_fixie = "span.is-$name-$i-fixie";
          if ($value['prefix']) {
            foreach ($value['prefix'] as $line) {
              $line = trim($line);
              $result = preg_replace('/\<is\>/', $element, $line);
              if ($result == $line) {
                $result .= ($result? " ": "").$element;
              }
              $css_selector .= ($css_selector? ',':'').$result;
              //for IE6
              if ($img['fixpngie']) {
                $result = preg_replace('/\<is\>/', $element_fixie, $line);
                if ($result == $line) {
                  $result .= ($result? " ": "").$element_fixie;
                }
                $css_selector_fixie .= ($css_selector_fixie? ',':'').$result;
              }
            }
          }
          $css .= "$css_selector{background-position:${ofsx}px ${ofsy}px;}\n";
          if ($img['fixpngie']) {
            $css_fixie .= "$css_selector_fixie{star:expression(IS_PNGFIX.calc_fixie(this,${ofsx},${ofsy},".($img['sizex']-$ofsx).",".($img['sizey']-$ofsy)."))}\n";
          }
        }
      }
    }
  }
	
  $css_url = "";
  $css_url_fixie = "";
  $path_css = file_create_path('imagesets');
  $count_root = explode('/', $path_css);	
  foreach($save_selectors as $value) {
    $img = _imageset_imageset($value['name']);
    $path = dirname($img['imgname']);
    if ($path == $path_css) { //current
      $url = basename($img['imgname']);
    }
    else {
      $url = $img['imgname'];
      for ($i = 1; $i <= count($count_root); $i++) {
        $url = "../".$url;
      }
    }
    $element = imageset_get_csselement($value['outname'], $value['name']);
    $css_url .= "$element{width:${img['sizex']}px; height:${img['sizey']}px;background:url($url) no-repeat transparent;}\n";
    if ($img['fixpngie']) {
      $element_fixie = "span.is-$name-fixie";
      $css_url_fixie .= "$element_fixie{position:relative;display:inline-block;background:transparent;}\n"; 
    }
  }
  return array('css' => $css_url.$css, 'css_fixie' => $css_url_fixie.$css_fixie);
}
